# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class FunctionScheduleEntryResponseBody(UniversalBaseModel):
    """
    A single schedule entry defining when a Function runs.
    """

    days_of_week: typing_extensions.Annotated[
        typing.List[bool],
        FieldMetadata(alias="daysOfWeek"),
        pydantic.Field(
            alias="daysOfWeek", description="Days of the week this entry applies to (7 elements, index 0 = Sunday)."
        ),
    ]
    time_since_midnight_ms: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="timeSinceMidnightMs"),
        pydantic.Field(
            alias="timeSinceMidnightMs", description="Time since midnight in milliseconds when the function runs."
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .alert_object_workforce_camera_device_response_body import AlertObjectWorkforceCameraDeviceResponseBody


class CameraStreamIssueResponseBody(UniversalBaseModel):
    """
    Details specific to Camera Stream Issue.
    """

    camera_device: typing_extensions.Annotated[
        typing.Optional[AlertObjectWorkforceCameraDeviceResponseBody],
        FieldMetadata(alias="cameraDevice"),
        pydantic.Field(alias="cameraDevice"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .tire_fault_code_details_object_request_body_manufacturer import TireFaultCodeDetailsObjectRequestBodyManufacturer
from .tire_fault_code_details_object_request_body_specific_tire_fault_codes_item import (
    TireFaultCodeDetailsObjectRequestBodySpecificTireFaultCodesItem,
)


class TireFaultCodeDetailsObjectRequestBody(UniversalBaseModel):
    """
    Details specific to Tire Fault Code. At least one fault code or fault code group must be selected.
    """

    has_cautionary_tire_fault_codes: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="hasCautionaryTireFaultCodes"),
        pydantic.Field(
            alias="hasCautionaryTireFaultCodes",
            description="If true then alert over pressure, under pressure, across axle fault, or leak detected fault codes. Defaults to false.",
        ),
    ] = None
    has_critical_tire_fault_codes: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="hasCriticalTireFaultCodes"),
        pydantic.Field(
            alias="hasCriticalTireFaultCodes",
            description="If true then alert over temperature or extreme pressure over or under fault codes. Defaults to false.",
        ),
    ] = None
    manufacturer: TireFaultCodeDetailsObjectRequestBodyManufacturer = pydantic.Field()
    """
    The tire manufacturer.  Valid values: `MANUFACTURER_BENDIX`, `MANUFACTURER_CONTINENTAL`, `MANUFACTURER_DORAN`, `MANUFACTURER_HENDRICKSON`, `MANUFACTURER_INVALID`, `MANUFACTURER_PRESSURE_PRO`, `MANUFACTURER_SENSATA`, `MANUFACTURER_UNIVERSAL_J1939`, `MANUFACTURER_UNIVERSAL_R141`
    """

    specific_tire_fault_codes: typing_extensions.Annotated[
        typing.Optional[typing.List[TireFaultCodeDetailsObjectRequestBodySpecificTireFaultCodesItem]],
        FieldMetadata(alias="specificTireFaultCodes"),
        pydantic.Field(
            alias="specificTireFaultCodes", description="The list of specific tire fault codes to be alerted on."
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .forms_media_record_object_response_body import FormsMediaRecordObjectResponseBody


class FormsMediaValueObjectResponseBody(UniversalBaseModel):
    """
    The value of a media form input field.
    """

    media_list: typing_extensions.Annotated[
        typing.List[FormsMediaRecordObjectResponseBody],
        FieldMetadata(alias="mediaList"),
        pydantic.Field(alias="mediaList", description="List of forms media record objects."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

MediaObjectResponseBodyStatus = typing.Union[
    typing.Literal["available", "invalid", "pending", "failed", "unavailable"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .engine_immobilizer_relay_state_response_body import EngineImmobilizerRelayStateResponseBody


class EngineImmobilizerStateResponseBody(UniversalBaseModel):
    """
    An engine immobilizer state.
    """

    happened_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="happenedAtTime"),
        pydantic.Field(
            alias="happenedAtTime",
            description="A UTC time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).",
        ),
    ]
    is_connected_to_vehicle: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isConnectedToVehicle"),
        pydantic.Field(
            alias="isConnectedToVehicle", description="Whether the engine immobilizer is connected the vehicle."
        ),
    ]
    relay_states: typing_extensions.Annotated[
        typing.List[EngineImmobilizerRelayStateResponseBody],
        FieldMetadata(alias="relayStates"),
        pydantic.Field(alias="relayStates", description="A list of states for each relay"),
    ]
    vehicle_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="vehicleId"),
        pydantic.Field(
            alias="vehicleId", description="The ID of the vehicle that the engine immobilizer is connected to."
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

WorkflowDvirObjectResponseBodyType = typing.Union[
    typing.Literal["preTrip", "postTrip", "mechanic", "unspecified"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .operational_settings_object_response_body_time_range_type import (
    OperationalSettingsObjectResponseBodyTimeRangeType,
)
from .time_range_object_response_body import TimeRangeObjectResponseBody


class OperationalSettingsObjectResponseBody(UniversalBaseModel):
    """
    Settings on when the alert should be operational.
    """

    time_range_type: typing_extensions.Annotated[
        OperationalSettingsObjectResponseBodyTimeRangeType,
        FieldMetadata(alias="timeRangeType"),
        pydantic.Field(
            alias="timeRangeType",
            description="The type of time ranges.  Valid values: `activeBetween`, `inactiveBetween`",
        ),
    ]
    time_ranges: typing_extensions.Annotated[
        typing.List[TimeRangeObjectResponseBody],
        FieldMetadata(alias="timeRanges"),
        pydantic.Field(alias="timeRanges", description="The time ranges this alert applies to."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

UpdateEngineImmobilizerRelayStateRequestBodyRequestBodyId = typing.Union[typing.Literal["relay1", "relay2"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .driver_with_external_id_object_response_body import DriverWithExternalIdObjectResponseBody
from .trailer_object_response_body import TrailerObjectResponseBody


class GetDriverTrailerAssignmentsResponseBodyResponseBody(UniversalBaseModel):
    """
    Object with driver assignment information and associated driver and trailer information.
    """

    created_at_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(
            alias="createdAtTime",
            description="Time when the driver trailer assignment was created, in RFC 3339 format.",
        ),
    ] = None
    driver: DriverWithExternalIdObjectResponseBody
    end_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="endTime"),
        pydantic.Field(
            alias="endTime", description="Time when the driver trailer assignment will end, in RFC 3339 format."
        ),
    ] = None
    id: str = pydantic.Field()
    """
    Samsara ID of the driver trailer assignment.
    """

    start_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="startTime"),
        pydantic.Field(
            alias="startTime", description="Time when the driver trailer assignment starts, in RFC 3339 format."
        ),
    ]
    trailer: TrailerObjectResponseBody
    updated_at_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(
            alias="updatedAtTime",
            description="Time when the driver trailer assignment was last updated, in RFC 3339 format.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .vehicle_stats_fault_codes_oem_trouble_code import VehicleStatsFaultCodesOemTroubleCode

VehicleStatsFaultCodesOemTroubleCodes = typing.List[VehicleStatsFaultCodesOemTroubleCode]



# This file was auto-generated by Fern from our API Definition.

import typing

HealthResponseResponseBodyRecommendedAction = typing.Union[
    typing.Literal[
        "recommendedActionAgHealthy",
        "recommendedActionAgLowDeviceBatteryAG45",
        "recommendedActionAgLowDeviceBatteryAG46",
        "recommendedActionAgLowDeviceBatteryAG51",
        "recommendedActionAgLowDeviceBatteryPoweredAG",
        "recommendedActionAgNoGpsSignal",
        "recommendedActionAgNotDetected",
        "recommendedActionAgUnplugged",
        "recommendedActionBLEDataPending",
        "recommendedActionBLEHealthy",
        "recommendedActionBLELowBattery",
        "recommendedActionBLENotDetected",
        "recommendedActionCcHealthy",
        "recommendedActionCcLowUptime",
        "recommendedActionCcMediaInputUptime",
        "recommendedActionCcNotDetected",
        "recommendedActionCmCameraMisaligned",
        "recommendedActionCmHealthy",
        "recommendedActionCmInactive",
        "recommendedActionCmInwardCameraObstruction",
        "recommendedActionCmIrregularRecording",
        "recommendedActionCmLowUptime",
        "recommendedActionCmNewlyInstalledDevice",
        "recommendedActionCmNotDetected",
        "recommendedActionCmOutwardCameraObstruction",
        "recommendedActionCmRecordingTimeRequired",
        "recommendedActionCmVgUnplugged",
        "recommendedActionGatewayNewlyInstalledDevice",
        "recommendedActionHealthy",
        "recommendedActionLowBatteryVoltage",
        "recommendedActionOemNotReporting",
        "recommendedActionVehicleOff",
        "recommendedActionVgHealthy",
        "recommendedActionVgMissingEldDiagnostics",
        "recommendedActionVgMissingEldDiagnosticsExchangeSubmitted",
        "recommendedActionVgMissingEldDiagnosticsHardwareExchanged",
        "recommendedActionVgMissingEldDiagnosticsNoExchangeSubmitted",
        "recommendedActionVgMissingVin",
        "recommendedActionVgNeedsReplacement",
        "recommendedActionVgNotDetected",
        "recommendedActionVgVgUnplugged",
        "unknown",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .address_geofence_polygon_vertices import AddressGeofencePolygonVertices


class AddressGeofencePolygon(UniversalBaseModel):
    """
    Information about a polygon geofence. This field is only needed if the geofence is a polygon.
    """

    vertices: typing.List[AddressGeofencePolygonVertices] = pydantic.Field()
    """
    The vertices of the polygon geofence. These geofence vertices describe the perimeter of the polygon, and must consist of at least 3 vertices and less than 40.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class InvoiceScanFileRequestBody(UniversalBaseModel):
    """
    Invoice file object
    """

    base_64_content: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="base64Content"),
        pydantic.Field(alias="base64Content", description="Base64 encoded file content. Maximum decoded size: 10MB."),
    ]
    content_type: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="contentType"),
        pydantic.Field(
            alias="contentType",
            description="MIME type of the file. Supported types: application/pdf, image/jpeg, image/png.",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .update_vehicle_request_aux_input_type_1 import UpdateVehicleRequestAuxInputType1
    from .update_vehicle_request_aux_input_type_10 import UpdateVehicleRequestAuxInputType10
    from .update_vehicle_request_aux_input_type_11 import UpdateVehicleRequestAuxInputType11
    from .update_vehicle_request_aux_input_type_12 import UpdateVehicleRequestAuxInputType12
    from .update_vehicle_request_aux_input_type_13 import UpdateVehicleRequestAuxInputType13
    from .update_vehicle_request_aux_input_type_2 import UpdateVehicleRequestAuxInputType2
    from .update_vehicle_request_aux_input_type_3 import UpdateVehicleRequestAuxInputType3
    from .update_vehicle_request_aux_input_type_4 import UpdateVehicleRequestAuxInputType4
    from .update_vehicle_request_aux_input_type_5 import UpdateVehicleRequestAuxInputType5
    from .update_vehicle_request_aux_input_type_6 import UpdateVehicleRequestAuxInputType6
    from .update_vehicle_request_aux_input_type_7 import UpdateVehicleRequestAuxInputType7
    from .update_vehicle_request_aux_input_type_8 import UpdateVehicleRequestAuxInputType8
    from .update_vehicle_request_aux_input_type_9 import UpdateVehicleRequestAuxInputType9
    from .update_vehicle_request_harsh_acceleration_setting_type import UpdateVehicleRequestHarshAccelerationSettingType
_dynamic_imports: typing.Dict[str, str] = {
    "UpdateVehicleRequestAuxInputType1": ".update_vehicle_request_aux_input_type_1",
    "UpdateVehicleRequestAuxInputType10": ".update_vehicle_request_aux_input_type_10",
    "UpdateVehicleRequestAuxInputType11": ".update_vehicle_request_aux_input_type_11",
    "UpdateVehicleRequestAuxInputType12": ".update_vehicle_request_aux_input_type_12",
    "UpdateVehicleRequestAuxInputType13": ".update_vehicle_request_aux_input_type_13",
    "UpdateVehicleRequestAuxInputType2": ".update_vehicle_request_aux_input_type_2",
    "UpdateVehicleRequestAuxInputType3": ".update_vehicle_request_aux_input_type_3",
    "UpdateVehicleRequestAuxInputType4": ".update_vehicle_request_aux_input_type_4",
    "UpdateVehicleRequestAuxInputType5": ".update_vehicle_request_aux_input_type_5",
    "UpdateVehicleRequestAuxInputType6": ".update_vehicle_request_aux_input_type_6",
    "UpdateVehicleRequestAuxInputType7": ".update_vehicle_request_aux_input_type_7",
    "UpdateVehicleRequestAuxInputType8": ".update_vehicle_request_aux_input_type_8",
    "UpdateVehicleRequestAuxInputType9": ".update_vehicle_request_aux_input_type_9",
    "UpdateVehicleRequestHarshAccelerationSettingType": ".update_vehicle_request_harsh_acceleration_setting_type",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "UpdateVehicleRequestAuxInputType1",
    "UpdateVehicleRequestAuxInputType10",
    "UpdateVehicleRequestAuxInputType11",
    "UpdateVehicleRequestAuxInputType12",
    "UpdateVehicleRequestAuxInputType13",
    "UpdateVehicleRequestAuxInputType2",
    "UpdateVehicleRequestAuxInputType3",
    "UpdateVehicleRequestAuxInputType4",
    "UpdateVehicleRequestAuxInputType5",
    "UpdateVehicleRequestAuxInputType6",
    "UpdateVehicleRequestAuxInputType7",
    "UpdateVehicleRequestAuxInputType8",
    "UpdateVehicleRequestAuxInputType9",
    "UpdateVehicleRequestHarshAccelerationSettingType",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_gateway_error import BadGatewayError
from ..errors.gateway_timeout_error import GatewayTimeoutError
from ..errors.internal_server_error import InternalServerError
from ..errors.method_not_allowed_error import MethodNotAllowedError
from ..errors.not_found_error import NotFoundError
from ..errors.not_implemented_error import NotImplementedError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.action_object_request_body import ActionObjectRequestBody
from ..types.alerts_get_configurations_response_body import AlertsGetConfigurationsResponseBody
from ..types.alerts_get_incidents_response_body import AlertsGetIncidentsResponseBody
from ..types.alerts_patch_configurations_response_body import AlertsPatchConfigurationsResponseBody
from ..types.alerts_post_configurations_response_body import AlertsPostConfigurationsResponseBody
from ..types.operational_settings_object_request_body import OperationalSettingsObjectRequestBody
from ..types.scope_object_request_body import ScopeObjectRequestBody
from ..types.workflow_trigger_object_request_body import WorkflowTriggerObjectRequestBody
from .types.get_configurations_request_status import GetConfigurationsRequestStatus

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawAlertsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_configurations(
        self,
        *,
        ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        status: typing.Optional[GetConfigurationsRequestStatus] = None,
        after: typing.Optional[str] = None,
        include_external_ids: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AlertsGetConfigurationsResponseBody]:
        """
        Get specified Alert Configurations.

        The following trigger types are API enabled and will show up in the results:
        Vehicle Speed
        Ambient Temperature
        Fuel Level (Percentage)
        Vehicle DEF Level (Percentage)
        Vehicle Battery
        Gateway Unplugged
        Dashcam Disconnected
        Camera Connector Disconnected
        Asset starts moving
        Inside Geofence
        Outside Geofence
        Unassigned Driving
        Driver HOS Violation
        Vehicle Engine Idle
        Asset Engine On
        Asset Engine Off
        Harsh Event
        Scheduled Maintenance
        Scheduled Maintenance by Odometer
        Scheduled Maintenance by Engine Hours
        Out of Route
        GPS Signal Loss
        Cell Signal Loss
        Fault Code
        Tire Faults
        Gateway Disconnected
        Panic Button
        Tampering Detected
        Asset Reading
        If vehicle is severely speeding (as defined by your organization)
        DVIR Submitted for Asset
        Driver Document Submitted
        Driver App Sign In
        Driver App Sign Out
        Geofence Entry
        Geofence Exit
        Route Stop ETA Alert
        Driver Recorded
        A safety event occurred with a driver assigned
        A safety event occurred
        Sudden Fuel Level Rise
        Sudden Fuel Level Drop
        Scheduled Date And Time
        Training Assignment Due Soon
        Training Assignment Past Due

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Alerts** under the Alerts category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter by the IDs. Returns all if no ids are provided.

        status : typing.Optional[GetConfigurationsRequestStatus]
            The status of the alert configuration.  Valid values: `all`, `enabled`, `disabled`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        include_external_ids : typing.Optional[bool]
            Optional boolean indicating whether to return external IDs on supported entities

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AlertsGetConfigurationsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "alerts/configurations",
            method="GET",
            params={
                "ids": ids,
                "status": status,
                "after": after,
                "includeExternalIds": include_external_ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AlertsGetConfigurationsResponseBody,
                    parse_obj_as(
                        type_=AlertsGetConfigurationsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def post_configurations(
        self,
        *,
        actions: typing.Sequence[ActionObjectRequestBody],
        is_enabled: bool,
        name: str,
        scope: ScopeObjectRequestBody,
        triggers: typing.Sequence[WorkflowTriggerObjectRequestBody],
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        operational_settings: typing.Optional[OperationalSettingsObjectRequestBody] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AlertsPostConfigurationsResponseBody]:
        """
        Creates an alert configuration.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Alerts** under the Alerts category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        actions : typing.Sequence[ActionObjectRequestBody]
            An array of actions.

        is_enabled : bool
            Whether the alert is enabled or not.

        name : str
            The custom name of the configuration.

        scope : ScopeObjectRequestBody

        triggers : typing.Sequence[WorkflowTriggerObjectRequestBody]
            An array of triggers.

        external_ids : typing.Optional[typing.Dict[str, str]]
            A map of external ids

        operational_settings : typing.Optional[OperationalSettingsObjectRequestBody]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AlertsPostConfigurationsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "alerts/configurations",
            method="POST",
            json={
                "actions": convert_and_respect_annotation_metadata(
                    object_=actions, annotation=typing.Sequence[ActionObjectRequestBody], direction="write"
                ),
                "externalIds": external_ids,
                "isEnabled": is_enabled,
                "name": name,
                "operationalSettings": convert_and_respect_annotation_metadata(
                    object_=operational_settings, annotation=OperationalSettingsObjectRequestBody, direction="write"
                ),
                "scope": convert_and_respect_annotation_metadata(
                    object_=scope, annotation=ScopeObjectRequestBody, direction="write"
                ),
                "triggers": convert_and_respect_annotation_metadata(
                    object_=triggers, annotation=typing.Sequence[WorkflowTriggerObjectRequestBody], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AlertsPostConfigurationsResponseBody,
                    parse_obj_as(
                        type_=AlertsPostConfigurationsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_configurations(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Delete an alert configuration.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Alerts** under the Alerts category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            The unqiue Samsara id of the alert configuration.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            "alerts/configurations",
            method="DELETE",
            params={
                "id": id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def patch_configurations(
        self,
        *,
        id: str,
        actions: typing.Optional[typing.Sequence[ActionObjectRequestBody]] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        is_enabled: typing.Optional[bool] = OMIT,
        name: typing.Optional[str] = OMIT,
        operational_settings: typing.Optional[OperationalSettingsObjectRequestBody] = OMIT,
        scope: typing.Optional[ScopeObjectRequestBody] = OMIT,
        triggers: typing.Optional[typing.Sequence[WorkflowTriggerObjectRequestBody]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AlertsPatchConfigurationsResponseBody]:
        """
        Updates an alert configuration.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Alerts** under the Alerts category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            The unqiue Samsara id of the alert configuration.

        actions : typing.Optional[typing.Sequence[ActionObjectRequestBody]]
            An array of actions.

        external_ids : typing.Optional[typing.Dict[str, str]]
            A map of external ids

        is_enabled : typing.Optional[bool]
            Whether the alert is enabled or not.

        name : typing.Optional[str]
            The custom name of the configuration.

        operational_settings : typing.Optional[OperationalSettingsObjectRequestBody]

        scope : typing.Optional[ScopeObjectRequestBody]

        triggers : typing.Optional[typing.Sequence[WorkflowTriggerObjectRequestBody]]
            An array of triggers.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AlertsPatchConfigurationsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "alerts/configurations",
            method="PATCH",
            json={
                "actions": convert_and_respect_annotation_metadata(
                    object_=actions, annotation=typing.Sequence[ActionObjectRequestBody], direction="write"
                ),
                "externalIds": external_ids,
                "id": id,
                "isEnabled": is_enabled,
                "name": name,
                "operationalSettings": convert_and_respect_annotation_metadata(
                    object_=operational_settings, annotation=OperationalSettingsObjectRequestBody, direction="write"
                ),
                "scope": convert_and_respect_annotation_metadata(
                    object_=scope, annotation=ScopeObjectRequestBody, direction="write"
                ),
                "triggers": convert_and_respect_annotation_metadata(
                    object_=triggers, annotation=typing.Sequence[WorkflowTriggerObjectRequestBody], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AlertsPatchConfigurationsResponseBody,
                    parse_obj_as(
                        type_=AlertsPatchConfigurationsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_incidents(
        self,
        *,
        start_time: str,
        configuration_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        end_time: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AlertsGetIncidentsResponseBody]:
        """
        Get Alert Incidents for specific Alert Configurations over a specified period of time.

         <b>Rate limit:</b> 10 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Alerts** under the Alerts category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            Required RFC 3339 timestamp that indicates when to begin receiving data. This will be based on updatedAtTime.

        configuration_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Required array of alert configuration ids to return incident data for.

        end_time : typing.Optional[str]
            Optional RFC 3339 timestamp to stop receiving data. Defaults to now if not provided. This will be based on updatedAtTime.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AlertsGetIncidentsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "alerts/incidents/stream",
            method="GET",
            params={
                "startTime": start_time,
                "configurationIds": configuration_ids,
                "endTime": end_time,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AlertsGetIncidentsResponseBody,
                    parse_obj_as(
                        type_=AlertsGetIncidentsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawAlertsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_configurations(
        self,
        *,
        ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        status: typing.Optional[GetConfigurationsRequestStatus] = None,
        after: typing.Optional[str] = None,
        include_external_ids: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AlertsGetConfigurationsResponseBody]:
        """
        Get specified Alert Configurations.

        The following trigger types are API enabled and will show up in the results:
        Vehicle Speed
        Ambient Temperature
        Fuel Level (Percentage)
        Vehicle DEF Level (Percentage)
        Vehicle Battery
        Gateway Unplugged
        Dashcam Disconnected
        Camera Connector Disconnected
        Asset starts moving
        Inside Geofence
        Outside Geofence
        Unassigned Driving
        Driver HOS Violation
        Vehicle Engine Idle
        Asset Engine On
        Asset Engine Off
        Harsh Event
        Scheduled Maintenance
        Scheduled Maintenance by Odometer
        Scheduled Maintenance by Engine Hours
        Out of Route
        GPS Signal Loss
        Cell Signal Loss
        Fault Code
        Tire Faults
        Gateway Disconnected
        Panic Button
        Tampering Detected
        Asset Reading
        If vehicle is severely speeding (as defined by your organization)
        DVIR Submitted for Asset
        Driver Document Submitted
        Driver App Sign In
        Driver App Sign Out
        Geofence Entry
        Geofence Exit
        Route Stop ETA Alert
        Driver Recorded
        A safety event occurred with a driver assigned
        A safety event occurred
        Sudden Fuel Level Rise
        Sudden Fuel Level Drop
        Scheduled Date And Time
        Training Assignment Due Soon
        Training Assignment Past Due

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Alerts** under the Alerts category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter by the IDs. Returns all if no ids are provided.

        status : typing.Optional[GetConfigurationsRequestStatus]
            The status of the alert configuration.  Valid values: `all`, `enabled`, `disabled`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        include_external_ids : typing.Optional[bool]
            Optional boolean indicating whether to return external IDs on supported entities

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AlertsGetConfigurationsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "alerts/configurations",
            method="GET",
            params={
                "ids": ids,
                "status": status,
                "after": after,
                "includeExternalIds": include_external_ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AlertsGetConfigurationsResponseBody,
                    parse_obj_as(
                        type_=AlertsGetConfigurationsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def post_configurations(
        self,
        *,
        actions: typing.Sequence[ActionObjectRequestBody],
        is_enabled: bool,
        name: str,
        scope: ScopeObjectRequestBody,
        triggers: typing.Sequence[WorkflowTriggerObjectRequestBody],
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        operational_settings: typing.Optional[OperationalSettingsObjectRequestBody] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AlertsPostConfigurationsResponseBody]:
        """
        Creates an alert configuration.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Alerts** under the Alerts category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        actions : typing.Sequence[ActionObjectRequestBody]
            An array of actions.

        is_enabled : bool
            Whether the alert is enabled or not.

        name : str
            The custom name of the configuration.

        scope : ScopeObjectRequestBody

        triggers : typing.Sequence[WorkflowTriggerObjectRequestBody]
            An array of triggers.

        external_ids : typing.Optional[typing.Dict[str, str]]
            A map of external ids

        operational_settings : typing.Optional[OperationalSettingsObjectRequestBody]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AlertsPostConfigurationsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "alerts/configurations",
            method="POST",
            json={
                "actions": convert_and_respect_annotation_metadata(
                    object_=actions, annotation=typing.Sequence[ActionObjectRequestBody], direction="write"
                ),
                "externalIds": external_ids,
                "isEnabled": is_enabled,
                "name": name,
                "operationalSettings": convert_and_respect_annotation_metadata(
                    object_=operational_settings, annotation=OperationalSettingsObjectRequestBody, direction="write"
                ),
                "scope": convert_and_respect_annotation_metadata(
                    object_=scope, annotation=ScopeObjectRequestBody, direction="write"
                ),
                "triggers": convert_and_respect_annotation_metadata(
                    object_=triggers, annotation=typing.Sequence[WorkflowTriggerObjectRequestBody], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AlertsPostConfigurationsResponseBody,
                    parse_obj_as(
                        type_=AlertsPostConfigurationsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_configurations(
        self, *, id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete an alert configuration.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Alerts** under the Alerts category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            The unqiue Samsara id of the alert configuration.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "alerts/configurations",
            method="DELETE",
            params={
                "id": id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def patch_configurations(
        self,
        *,
        id: str,
        actions: typing.Optional[typing.Sequence[ActionObjectRequestBody]] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        is_enabled: typing.Optional[bool] = OMIT,
        name: typing.Optional[str] = OMIT,
        operational_settings: typing.Optional[OperationalSettingsObjectRequestBody] = OMIT,
        scope: typing.Optional[ScopeObjectRequestBody] = OMIT,
        triggers: typing.Optional[typing.Sequence[WorkflowTriggerObjectRequestBody]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AlertsPatchConfigurationsResponseBody]:
        """
        Updates an alert configuration.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Alerts** under the Alerts category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            The unqiue Samsara id of the alert configuration.

        actions : typing.Optional[typing.Sequence[ActionObjectRequestBody]]
            An array of actions.

        external_ids : typing.Optional[typing.Dict[str, str]]
            A map of external ids

        is_enabled : typing.Optional[bool]
            Whether the alert is enabled or not.

        name : typing.Optional[str]
            The custom name of the configuration.

        operational_settings : typing.Optional[OperationalSettingsObjectRequestBody]

        scope : typing.Optional[ScopeObjectRequestBody]

        triggers : typing.Optional[typing.Sequence[WorkflowTriggerObjectRequestBody]]
            An array of triggers.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AlertsPatchConfigurationsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "alerts/configurations",
            method="PATCH",
            json={
                "actions": convert_and_respect_annotation_metadata(
                    object_=actions, annotation=typing.Sequence[ActionObjectRequestBody], direction="write"
                ),
                "externalIds": external_ids,
                "id": id,
                "isEnabled": is_enabled,
                "name": name,
                "operationalSettings": convert_and_respect_annotation_metadata(
                    object_=operational_settings, annotation=OperationalSettingsObjectRequestBody, direction="write"
                ),
                "scope": convert_and_respect_annotation_metadata(
                    object_=scope, annotation=ScopeObjectRequestBody, direction="write"
                ),
                "triggers": convert_and_respect_annotation_metadata(
                    object_=triggers, annotation=typing.Sequence[WorkflowTriggerObjectRequestBody], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AlertsPatchConfigurationsResponseBody,
                    parse_obj_as(
                        type_=AlertsPatchConfigurationsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_incidents(
        self,
        *,
        start_time: str,
        configuration_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        end_time: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AlertsGetIncidentsResponseBody]:
        """
        Get Alert Incidents for specific Alert Configurations over a specified period of time.

         <b>Rate limit:</b> 10 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Alerts** under the Alerts category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            Required RFC 3339 timestamp that indicates when to begin receiving data. This will be based on updatedAtTime.

        configuration_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Required array of alert configuration ids to return incident data for.

        end_time : typing.Optional[str]
            Optional RFC 3339 timestamp to stop receiving data. Defaults to now if not provided. This will be based on updatedAtTime.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AlertsGetIncidentsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "alerts/incidents/stream",
            method="GET",
            params={
                "startTime": start_time,
                "configurationIds": configuration_ids,
                "endTime": end_time,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AlertsGetIncidentsResponseBody,
                    parse_obj_as(
                        type_=AlertsGetIncidentsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class QrCodeResponseObjectResponseBody(UniversalBaseModel):
    """
    A single document.
    """

    driver_id: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="driverId"),
        pydantic.Field(alias="driverId", description="ID for the driver the QR code belongs to."),
    ]
    qr_code_link: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="qrCodeLink"),
        pydantic.Field(
            alias="qrCodeLink",
            description="URL link to the driver assignment QR code. Included if a QR code has been created for the driver.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

SafetyScoreBehaviorObjectResponseBodyBehaviorType = typing.Union[
    typing.Literal[
        "acceleration",
        "braking",
        "crash",
        "defensiveDriving",
        "didNotYield",
        "distractedDrivingAutomatic",
        "distractedDrivingManual",
        "drowsy",
        "eatingDrinking",
        "followingDistance",
        "followingDistanceModerate",
        "followingDistanceSevere",
        "forwardCollisionWarning",
        "harshTurn",
        "laneDeparture",
        "lateResponse",
        "mobileUsage",
        "nearCollision",
        "noSeatbelt",
        "obstructedCamera",
        "ranRedLight",
        "rollingStop",
        "smoking",
        "speedingManual",
        "unknown",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_gateway_error import BadGatewayError
from ..errors.gateway_timeout_error import GatewayTimeoutError
from ..errors.internal_server_error import InternalServerError
from ..errors.method_not_allowed_error import MethodNotAllowedError
from ..errors.not_found_error import NotFoundError
from ..errors.not_implemented_error import NotImplementedError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.trips_get_trips_response_body import TripsGetTripsResponseBody
from ..types.v_1_trip_response import V1TripResponse
from .types.get_trips_request_completion_status import GetTripsRequestCompletionStatus
from .types.get_trips_request_query_by import GetTripsRequestQueryBy


class RawTripsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_trips(
        self,
        *,
        start_time: str,
        include_asset: typing.Optional[bool] = None,
        completion_status: typing.Optional[GetTripsRequestCompletionStatus] = None,
        end_time: typing.Optional[str] = None,
        query_by: typing.Optional[GetTripsRequestQueryBy] = None,
        after: typing.Optional[str] = None,
        ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TripsGetTripsResponseBody]:
        """
        This endpoint will return trips that have been collected for your organization based on the time parameters passed in. Results are paginated.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Trips** under the Trips category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            RFC 3339 timestamp that indicates when to begin receiving data. Value is compared against `updatedAtTime` or `tripStartTime` depending on the queryBy parameter.

        include_asset : typing.Optional[bool]
            Indicates whether or not to return expanded “asset” data

        completion_status : typing.Optional[GetTripsRequestCompletionStatus]
            Filters trips based on a specific completion status  Valid values: `inProgress`, `completed`, `all`

        end_time : typing.Optional[str]
            RFC 3339 timestamp which is compared against `updatedAtTime` or `tripStartTime` depending on the queryBy parameter. If not provided then the endpoint behaves as an unending feed of changes.

        query_by : typing.Optional[GetTripsRequestQueryBy]
            Decide which timestamp the `startTime` and `endTime` are compared to.  Valid values: `updatedAtTime`, `tripStartTime`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Comma-separated list of asset IDs. Include up to 50 asset IDs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TripsGetTripsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "trips/stream",
            method="GET",
            params={
                "includeAsset": include_asset,
                "completionStatus": completion_status,
                "startTime": start_time,
                "endTime": end_time,
                "queryBy": query_by,
                "after": after,
                "ids": ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TripsGetTripsResponseBody,
                    parse_obj_as(
                        type_=TripsGetTripsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def v_1_get_fleet_trips(
        self, *, vehicle_id: int, start_ms: int, end_ms: int, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[V1TripResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get historical trips data for specified vehicle. This method returns a set of historical trips data for the specified vehicle in the specified time range.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Vehicle Trips** under the Vehicles category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        vehicle_id : int
            Vehicle ID to query.

        start_ms : int
            Beginning of the time range, specified in milliseconds UNIX time. Limited to a 90 day window with respect to startMs and endMs

        end_ms : int
            End of the time range, specified in milliseconds UNIX time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[V1TripResponse]
            List of trips taken by the requested vehicle within the specified timeframe. Ongoing trips will be returned with 9223372036854775807 as their endMs.
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/fleet/trips",
            method="GET",
            params={
                "vehicleId": vehicle_id,
                "startMs": start_ms,
                "endMs": end_ms,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1TripResponse,
                    parse_obj_as(
                        type_=V1TripResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTripsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_trips(
        self,
        *,
        start_time: str,
        include_asset: typing.Optional[bool] = None,
        completion_status: typing.Optional[GetTripsRequestCompletionStatus] = None,
        end_time: typing.Optional[str] = None,
        query_by: typing.Optional[GetTripsRequestQueryBy] = None,
        after: typing.Optional[str] = None,
        ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TripsGetTripsResponseBody]:
        """
        This endpoint will return trips that have been collected for your organization based on the time parameters passed in. Results are paginated.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Trips** under the Trips category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            RFC 3339 timestamp that indicates when to begin receiving data. Value is compared against `updatedAtTime` or `tripStartTime` depending on the queryBy parameter.

        include_asset : typing.Optional[bool]
            Indicates whether or not to return expanded “asset” data

        completion_status : typing.Optional[GetTripsRequestCompletionStatus]
            Filters trips based on a specific completion status  Valid values: `inProgress`, `completed`, `all`

        end_time : typing.Optional[str]
            RFC 3339 timestamp which is compared against `updatedAtTime` or `tripStartTime` depending on the queryBy parameter. If not provided then the endpoint behaves as an unending feed of changes.

        query_by : typing.Optional[GetTripsRequestQueryBy]
            Decide which timestamp the `startTime` and `endTime` are compared to.  Valid values: `updatedAtTime`, `tripStartTime`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Comma-separated list of asset IDs. Include up to 50 asset IDs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TripsGetTripsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "trips/stream",
            method="GET",
            params={
                "includeAsset": include_asset,
                "completionStatus": completion_status,
                "startTime": start_time,
                "endTime": end_time,
                "queryBy": query_by,
                "after": after,
                "ids": ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TripsGetTripsResponseBody,
                    parse_obj_as(
                        type_=TripsGetTripsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def v_1_get_fleet_trips(
        self, *, vehicle_id: int, start_ms: int, end_ms: int, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[V1TripResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get historical trips data for specified vehicle. This method returns a set of historical trips data for the specified vehicle in the specified time range.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Vehicle Trips** under the Vehicles category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        vehicle_id : int
            Vehicle ID to query.

        start_ms : int
            Beginning of the time range, specified in milliseconds UNIX time. Limited to a 90 day window with respect to startMs and endMs

        end_ms : int
            End of the time range, specified in milliseconds UNIX time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[V1TripResponse]
            List of trips taken by the requested vehicle within the specified timeframe. Ongoing trips will be returned with 9223372036854775807 as their endMs.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/fleet/trips",
            method="GET",
            params={
                "vehicleId": vehicle_id,
                "startMs": start_ms,
                "endMs": end_ms,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1TripResponse,
                    parse_obj_as(
                        type_=V1TripResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .parent_tag_name import ParentTagName
from .tagged_object_id import TaggedObjectId


class ParentTag(UniversalBaseModel):
    """
    If this tag is part a hierarchical tag tree, this is the parent tag, otherwise this will be omitted.
    """

    id: TaggedObjectId
    name: typing.Optional[ParentTagName] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.request_options import RequestOptions


class RawCarrierProposedAssignmentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Permanently delete an assignment. You can only delete assignments that are not yet active. To override a currently active assignment, create a new empty one, instead.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Carrier-Proposed Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the assignment.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"fleet/carrier-proposed-assignments/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawCarrierProposedAssignmentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Permanently delete an assignment. You can only delete assignments that are not yet active. To override a currently active assignment, create a new empty one, instead.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Carrier-Proposed Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the assignment.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"fleet/carrier-proposed-assignments/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .asset_data_input_last_point import AssetDataInputLastPoint


class AssetDataInput(UniversalBaseModel):
    data_group: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="dataGroup"),
        pydantic.Field(alias="dataGroup", description="Name of the data group that the data input is associated with"),
    ] = None
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the data input
    """

    last_point: typing_extensions.Annotated[
        typing.Optional[AssetDataInputLastPoint], FieldMetadata(alias="lastPoint"), pydantic.Field(alias="lastPoint")
    ] = None
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the data input
    """

    units: typing.Optional[str] = pydantic.Field(default=None)
    """
    Units of data for this data input
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .trailer_stat_gps_odometer_meters_type_response_body import TrailerStatGpsOdometerMetersTypeResponseBody
from .trailer_stat_gps_type_response_body import TrailerStatGpsTypeResponseBody
from .trailer_stat_reefer_alarm_type_response_body import TrailerStatReeferAlarmTypeResponseBody
from .trailer_stat_reefer_ambient_air_temperature_milli_c_type_response_body import (
    TrailerStatReeferAmbientAirTemperatureMilliCTypeResponseBody,
)
from .trailer_stat_reefer_door_state_zone_1_type_response_body import TrailerStatReeferDoorStateZone1TypeResponseBody
from .trailer_stat_reefer_door_state_zone_2_type_response_body import TrailerStatReeferDoorStateZone2TypeResponseBody
from .trailer_stat_reefer_door_state_zone_3_type_response_body import TrailerStatReeferDoorStateZone3TypeResponseBody
from .trailer_stat_reefer_fuel_percent_type_response_body import TrailerStatReeferFuelPercentTypeResponseBody
from .trailer_stat_reefer_obd_engine_seconds_type_response_body import TrailerStatReeferObdEngineSecondsTypeResponseBody
from .trailer_stat_reefer_return_air_temperature_milli_c_zone_1_type_response_body import (
    TrailerStatReeferReturnAirTemperatureMilliCZone1TypeResponseBody,
)
from .trailer_stat_reefer_return_air_temperature_milli_c_zone_2_type_response_body import (
    TrailerStatReeferReturnAirTemperatureMilliCZone2TypeResponseBody,
)
from .trailer_stat_reefer_return_air_temperature_milli_c_zone_3_type_response_body import (
    TrailerStatReeferReturnAirTemperatureMilliCZone3TypeResponseBody,
)
from .trailer_stat_reefer_run_mode_type_response_body import TrailerStatReeferRunModeTypeResponseBody
from .trailer_stat_reefer_set_point_temperature_milli_c_zone_1_type_response_body import (
    TrailerStatReeferSetPointTemperatureMilliCZone1TypeResponseBody,
)
from .trailer_stat_reefer_set_point_temperature_milli_c_zone_2_type_response_body import (
    TrailerStatReeferSetPointTemperatureMilliCZone2TypeResponseBody,
)
from .trailer_stat_reefer_set_point_temperature_milli_c_zone_3_type_response_body import (
    TrailerStatReeferSetPointTemperatureMilliCZone3TypeResponseBody,
)
from .trailer_stat_reefer_state_type_response_body import TrailerStatReeferStateTypeResponseBody
from .trailer_stat_reefer_state_zone_1_type_response_body import TrailerStatReeferStateZone1TypeResponseBody
from .trailer_stat_reefer_state_zone_2_type_response_body import TrailerStatReeferStateZone2TypeResponseBody
from .trailer_stat_reefer_state_zone_3_type_response_body import TrailerStatReeferStateZone3TypeResponseBody
from .trailer_stat_reefer_supply_air_temperature_milli_c_zone_1_type_response_body import (
    TrailerStatReeferSupplyAirTemperatureMilliCZone1TypeResponseBody,
)
from .trailer_stat_reefer_supply_air_temperature_milli_c_zone_2_type_response_body import (
    TrailerStatReeferSupplyAirTemperatureMilliCZone2TypeResponseBody,
)
from .trailer_stat_reefer_supply_air_temperature_milli_c_zone_3_type_response_body import (
    TrailerStatReeferSupplyAirTemperatureMilliCZone3TypeResponseBody,
)


class TrailerStatsSnapshotObjectResponseBody(UniversalBaseModel):
    """
    A trailer and its stats
    """

    carrier_reefer_state: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferStateTypeResponseBody],
        FieldMetadata(alias="carrierReeferState"),
        pydantic.Field(alias="carrierReeferState"),
    ] = None
    gps: typing.Optional[TrailerStatGpsTypeResponseBody] = None
    gps_odometer_meters: typing_extensions.Annotated[
        typing.Optional[TrailerStatGpsOdometerMetersTypeResponseBody],
        FieldMetadata(alias="gpsOdometerMeters"),
        pydantic.Field(alias="gpsOdometerMeters"),
    ] = None
    id: str = pydantic.Field()
    """
    ID of the trailer.
    """

    name: str = pydantic.Field()
    """
    Name of the vehicle.
    """

    reefer_alarms: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferAlarmTypeResponseBody],
        FieldMetadata(alias="reeferAlarms"),
        pydantic.Field(alias="reeferAlarms"),
    ] = None
    reefer_ambient_air_temperature_milli_c: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferAmbientAirTemperatureMilliCTypeResponseBody],
        FieldMetadata(alias="reeferAmbientAirTemperatureMilliC"),
        pydantic.Field(alias="reeferAmbientAirTemperatureMilliC"),
    ] = None
    reefer_door_state_zone_1: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferDoorStateZone1TypeResponseBody],
        FieldMetadata(alias="reeferDoorStateZone1"),
        pydantic.Field(alias="reeferDoorStateZone1"),
    ] = None
    reefer_door_state_zone_2: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferDoorStateZone2TypeResponseBody],
        FieldMetadata(alias="reeferDoorStateZone2"),
        pydantic.Field(alias="reeferDoorStateZone2"),
    ] = None
    reefer_door_state_zone_3: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferDoorStateZone3TypeResponseBody],
        FieldMetadata(alias="reeferDoorStateZone3"),
        pydantic.Field(alias="reeferDoorStateZone3"),
    ] = None
    reefer_fuel_percent: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferFuelPercentTypeResponseBody],
        FieldMetadata(alias="reeferFuelPercent"),
        pydantic.Field(alias="reeferFuelPercent"),
    ] = None
    reefer_obd_engine_seconds: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferObdEngineSecondsTypeResponseBody],
        FieldMetadata(alias="reeferObdEngineSeconds"),
        pydantic.Field(alias="reeferObdEngineSeconds"),
    ] = None
    reefer_return_air_temperature_milli_c_zone_1: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferReturnAirTemperatureMilliCZone1TypeResponseBody],
        FieldMetadata(alias="reeferReturnAirTemperatureMilliCZone1"),
        pydantic.Field(alias="reeferReturnAirTemperatureMilliCZone1"),
    ] = None
    reefer_return_air_temperature_milli_c_zone_2: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferReturnAirTemperatureMilliCZone2TypeResponseBody],
        FieldMetadata(alias="reeferReturnAirTemperatureMilliCZone2"),
        pydantic.Field(alias="reeferReturnAirTemperatureMilliCZone2"),
    ] = None
    reefer_return_air_temperature_milli_c_zone_3: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferReturnAirTemperatureMilliCZone3TypeResponseBody],
        FieldMetadata(alias="reeferReturnAirTemperatureMilliCZone3"),
        pydantic.Field(alias="reeferReturnAirTemperatureMilliCZone3"),
    ] = None
    reefer_run_mode: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferRunModeTypeResponseBody],
        FieldMetadata(alias="reeferRunMode"),
        pydantic.Field(alias="reeferRunMode"),
    ] = None
    reefer_set_point_temperature_milli_c_zone_1: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferSetPointTemperatureMilliCZone1TypeResponseBody],
        FieldMetadata(alias="reeferSetPointTemperatureMilliCZone1"),
        pydantic.Field(alias="reeferSetPointTemperatureMilliCZone1"),
    ] = None
    reefer_set_point_temperature_milli_c_zone_2: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferSetPointTemperatureMilliCZone2TypeResponseBody],
        FieldMetadata(alias="reeferSetPointTemperatureMilliCZone2"),
        pydantic.Field(alias="reeferSetPointTemperatureMilliCZone2"),
    ] = None
    reefer_set_point_temperature_milli_c_zone_3: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferSetPointTemperatureMilliCZone3TypeResponseBody],
        FieldMetadata(alias="reeferSetPointTemperatureMilliCZone3"),
        pydantic.Field(alias="reeferSetPointTemperatureMilliCZone3"),
    ] = None
    reefer_state_zone_1: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferStateZone1TypeResponseBody],
        FieldMetadata(alias="reeferStateZone1"),
        pydantic.Field(alias="reeferStateZone1"),
    ] = None
    reefer_state_zone_2: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferStateZone2TypeResponseBody],
        FieldMetadata(alias="reeferStateZone2"),
        pydantic.Field(alias="reeferStateZone2"),
    ] = None
    reefer_state_zone_3: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferStateZone3TypeResponseBody],
        FieldMetadata(alias="reeferStateZone3"),
        pydantic.Field(alias="reeferStateZone3"),
    ] = None
    reefer_supply_air_temperature_milli_c_zone_1: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferSupplyAirTemperatureMilliCZone1TypeResponseBody],
        FieldMetadata(alias="reeferSupplyAirTemperatureMilliCZone1"),
        pydantic.Field(alias="reeferSupplyAirTemperatureMilliCZone1"),
    ] = None
    reefer_supply_air_temperature_milli_c_zone_2: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferSupplyAirTemperatureMilliCZone2TypeResponseBody],
        FieldMetadata(alias="reeferSupplyAirTemperatureMilliCZone2"),
        pydantic.Field(alias="reeferSupplyAirTemperatureMilliCZone2"),
    ] = None
    reefer_supply_air_temperature_milli_c_zone_3: typing_extensions.Annotated[
        typing.Optional[TrailerStatReeferSupplyAirTemperatureMilliCZone3TypeResponseBody],
        FieldMetadata(alias="reeferSupplyAirTemperatureMilliCZone3"),
        pydantic.Field(alias="reeferSupplyAirTemperatureMilliCZone3"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.gateways_get_gateways_response_body import GatewaysGetGatewaysResponseBody
from ..types.gateways_post_gateway_response_body import GatewaysPostGatewayResponseBody
from .raw_client import AsyncRawGatewaysClient, RawGatewaysClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class GatewaysClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawGatewaysClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawGatewaysClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawGatewaysClient
        """
        return self._raw_client

    def get_gateways(
        self,
        *,
        models: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GatewaysGetGatewaysResponseBody:
        """
        List all gateways

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Gateways** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        models : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter by a comma separated list of gateway models.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GatewaysGetGatewaysResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.gateways.get_gateways()
        """
        _response = self._raw_client.get_gateways(models=models, after=after, request_options=request_options)
        return _response.data

    def post_gateway(
        self, *, serial: str, request_options: typing.Optional[RequestOptions] = None
    ) -> GatewaysPostGatewayResponseBody:
        """
        Activate a new gateway. To activate a device and associate it with your organization, enter its serial number. Each device's serial number can also be found on its label or packaging, or from your order confirmation email. A Not Found error could mean that the serial was not found or it has already been activated

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Gateways** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        serial : str
            Gateway serial number

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GatewaysPostGatewayResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.gateways.post_gateway(
            serial="GFRV-43N-VGX",
        )
        """
        _response = self._raw_client.post_gateway(serial=serial, request_options=request_options)
        return _response.data

    def delete_gateway(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Deactivate a gateway

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Gateways** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            Gateway serial number

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.gateways.delete_gateway(
            id="id",
        )
        """
        _response = self._raw_client.delete_gateway(id, request_options=request_options)
        return _response.data


class AsyncGatewaysClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawGatewaysClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawGatewaysClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawGatewaysClient
        """
        return self._raw_client

    async def get_gateways(
        self,
        *,
        models: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GatewaysGetGatewaysResponseBody:
        """
        List all gateways

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Gateways** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        models : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter by a comma separated list of gateway models.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GatewaysGetGatewaysResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.gateways.get_gateways()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_gateways(models=models, after=after, request_options=request_options)
        return _response.data

    async def post_gateway(
        self, *, serial: str, request_options: typing.Optional[RequestOptions] = None
    ) -> GatewaysPostGatewayResponseBody:
        """
        Activate a new gateway. To activate a device and associate it with your organization, enter its serial number. Each device's serial number can also be found on its label or packaging, or from your order confirmation email. A Not Found error could mean that the serial was not found or it has already been activated

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Gateways** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        serial : str
            Gateway serial number

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GatewaysPostGatewayResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.gateways.post_gateway(
                serial="GFRV-43N-VGX",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.post_gateway(serial=serial, request_options=request_options)
        return _response.data

    async def delete_gateway(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Deactivate a gateway

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Gateways** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        id : str
            Gateway serial number

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.gateways.delete_gateway(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_gateway(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .engine_immobilizer_relay_state_response_body_id import EngineImmobilizerRelayStateResponseBodyId


class EngineImmobilizerRelayStateResponseBody(UniversalBaseModel):
    """
    An engine immobilizer relay state.
    """

    id: EngineImmobilizerRelayStateResponseBodyId = pydantic.Field()
    """
    The ID of the relay  Valid values: `relay1`, `relay2`
    """

    is_open: typing_extensions.Annotated[
        bool, FieldMetadata(alias="isOpen"), pydantic.Field(alias="isOpen", description="Whether the relay is open.")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
